﻿import com.delfiweb.command._Command;



/**
 * Un objet AppelFct permet d'appeler une commande avec un temps de latence entre 
 * la création de l'objet et son appel effectif. 
 * 
 * @author  Cédric Néhémie
 * @version 1.0
 * @see     _Command	
 */
class com.delfiweb.utils.AppelFct implements _Command
{
	
	
	private var _oCommand:_Command;
	private var _nInterval:Number;
	private var _nIntervalDuree:Number;
	private var _bIntervalEnCours:Boolean;
	
	
	
	/**
	 * Créer un objet AppelFct, Le premier paramètre est la commande a éxecuter, si ce paramètre est omis une
	 * erreur est renvoyée, autrement tous les autres paramètres sont optionnel et prendront des valeurs par 
	 * défauts si ils sont omis.
	 * <ul>
	 * <li>Si le paramètre <b>t</b> est omis : Il prend la valeur <b>1</b>, c'est-à-dire que la fonction sera appelée
	 * quasi intantanément.</li>
	 * <li>Si le paramètre <b>autoLaunch</b> est omis : Il prend la valeur <b>true</b>, c'est à dire que le décalage
	 * sera lancée dès la création de l'objet.</li>
	 * </ul>
	 * Pour utiliser cette classe pour appeler des fonctions, il suffit d'utiliser
	 * 
	 * utilisation :    // un AppelFct se lancant automatiquement
	 * var timer:AppelFct = new AppelFct(command,1000);
	 * utilisation :    // un AppelFct ne se lancant pas automatiquement
	 * var timer:AppelFct = new AppelFct(command,1000,false);
	 * entrée :    c:_Command	Une commande a appelée       
	 * entrée :    t:Number	La durée de latence avant l'appel de la commande en millisecondes.         
	 * entrée :    autoLaunch:Boolean  Lancement automatique de la commande lors de sa création.
	 * @throw	Error	Si aucune commande n'est transmise alors l'objet renvoi une erreur critique.
	 * sortie :   Void
	 */
	public function AppelFct (c:_Command,t:Number,autoLaunch:Boolean)
	{
		if(!c) 
		{
			var s = "Attention, vous tentez de créer un objet AppelFct sans préciser la commande a appelée en fin de cycle"
			trace (s);
			//throw new Error (s);
		}
		if(autoLaunch == undefined) autoLaunch = true;
		
		_oCommand = c;
		_nIntervalDuree = t ? t : 1;
		_bIntervalEnCours = false;
		
		if(autoLaunch)
			execute();
	}
	
	
	
	/**
	 * Méthode issue de l'implémentation de {@link com.delfiweb.command._Command}, c'est lors de l'appel
	 * de cette méthode que le compte à rebourd est déclenché, si on rappelle la méthode alors que le compte
	 * à rebourd n'est pas encore terminé l'action échoue et renvoi <b>false</b>.
	 * 
	 * utilisation :    command.execute();
	 * sortie :   
	 */
	public function execute():Void
	{
		if(!_bIntervalEnCours)
		{
			_nInterval = setInterval (this,"__callback",_nIntervalDuree);
			_bIntervalEnCours = true;
		}
		else
		{
			//return false;
		}
	}
	
	
	
	/**
	 * Permet d'interrompre le AppelFct durant le compte à rebours, comme par exemple pour
	 * by-passer une vérification ou autre.
	 * 
	 * utilisation :    command.abort();
	 * sortie :   Void
	 */
	public function abort ()
	{
		clearInterval (_nInterval);
		_bIntervalEnCours = false;
	}
	
	
	
	public function setCommand(o:_Command)
	{
		if(!_bIntervalEnCours)
		{
			_oCommand = o;
		}
	}
	
	

	public function getCommand ():_Command
	{
		return _oCommand;
	}
	
	

	public function setTiming(n:Number)
	{
		if(!_bIntervalEnCours)
		{
			_nIntervalDuree = n ? n : _nIntervalDuree;
		}
	}
	
	
	
	public function getTiming():Number
	{
		return _nIntervalDuree;
	}
	
	

	public function clone ()
	{
		return new AppelFct (_oCommand,_nIntervalDuree,false);
	}
	
	
	
	private function __callback ()
	{
		_oCommand.execute();
		abort ();
	}
	
	
}

